from pathlib import Path
from typing import Any, Dict, Sequence

from twig.backends import VyperBackend
from twig.utils.compiler import generate_contract_types, generate_inline_sources


class Compiler:
    def __init__(self, sources: Sequence[Path], backend: VyperBackend) -> None:
        self.sources = sources
        self.backend = backend
        self.output: Dict[str, Any] = None

    def compile(self) -> None:
        self.output = self.backend.compile(self.sources)

    def get_contract_types(self) -> Dict[str, Any]:
        if not self.output:
            self.compile()
        return generate_contract_types(self.output)

    def get_source_tree(self) -> Dict[str, str]:
        if not self.output:
            self.compile()
        return generate_inline_sources(self.output)



from typing import Any, Dict, Iterable

from eth_utils import to_tuple

from ethpm.tools import builder as b
from vyper import compile_code


@to_tuple
def generate_inline_sources(
    compiler_output: Dict[str, Any]
) -> Iterable[Dict[str, str]]:
    for path in compiler_output.keys():
        contract_type = path.split("/")[-1].split(".")[0]
        yield b.inline_source(contract_type, compiler_output)


@to_tuple
def generate_contract_types(
    compiler_output: Dict[str, Any]
) -> Iterable[Dict[str, Any]]:
    for path in compiler_output.keys():
        contract_type = path.split("/")[-1].split(".")[0]
        yield b.contract_type(contract_type, compiler_output)


def create_raw_asset_data(source: str) -> Dict[str, Any]:
    out = compile_code(source, ["bytecode", "abi"])
    return {
        "abi": out["abi"],
        "evm": {"bytecode": {"object": out["bytecode"], "linkReferences": {}}},
    }



from pathlib import Path
from typing import Any, Dict, Iterable, Sequence, Tuple

from eth_utils import to_dict

from twig.utils.compiler import create_raw_asset_data


class VyperBackend:
    def compile(self, sources: Sequence[Path]) -> Dict[str, Any]:
        return generate_vyper_compiler_output(sources)


@to_dict
def generate_vyper_compiler_output(
    all_sources: Sequence[Path]
) -> Iterable[Tuple[str, Any]]:
    for source in all_sources:
        contract_file = str(source).split("/")[-1]
        contract_type = contract_file.split(".")[0]
        # todo fix to accomodate multiple types in a single contract file.
        yield str(source), {contract_type: create_raw_asset_data(source.read_text())}



from pathlib import Path
from typing import Iterable

SOURCES_GLOB = "**/*.vy"


def collect_sources(path: Path, glob: str = SOURCES_GLOB) -> Iterable[Path]:
    all_sources = Path(path).glob(glob)
    return all_sources
